# import uuid

from django.db import models
from django.urls import reverse
from django.contrib.auth.models import User


class Fish(models.Model):
    # Fields
    species_name = models.CharField(max_length=21)
    fish_length = models.FloatField()
    fish_weight = models.FloatField()
    date_caught = models.DateTimeField(auto_now=False, auto_now_add=False)
    gps_coordinates = models.CharField(max_length=40)
    other_comments = models.TextField(max_length=100, null=True, blank=True)
    upload = models.ForeignKey('Upload', on_delete=models.CASCADE)

    # So pycharm doesn't throw warning on using objects.all
    objects = models.Manager()

    # Metadata
    class Meta:
        ordering = ['-species_name']

    # Methods
    def __str__(self):
        return self.species_name

    def get_uploader(self):
        return self.upload.uploader

    def get_absolute_url(self):
        return reverse('model-detail-view', args=[str(self.id)])


class Upload(models.Model):
    # Fields
    timestamp = models.DateTimeField(auto_now=False, auto_now_add=True, primary_key=True)
    uploader = models.ForeignKey(User, on_delete=models.SET_NULL, null=True, blank=True)
    public = models.BooleanField()
    TYPES = (
        ('e', 'excel'),
        ('w', 'word'),
        ('p', 'pdf'),
    )
    file_type = models.CharField(max_length=5, choices=TYPES, default='e')
    file = models.FileField(upload_to='Uploads/')
    # https://docs.djangoproject.com/en/3.1/ref/models/fields/#filefield

    # So pycharm doesn't throw warning on using objects.all
    objects = models.Manager()

    # Metadata
    class Meta:
        ordering = ['-timestamp']

    # Methods
    def __str__(self):
        return str(self.timestamp)

    def get_uploader(self):
        return self.uploader

    def get_absolute_url(self):
        return reverse('model-detail-view', args=[str(self.timestamp)])
