from django.shortcuts import render
from table.models import Fish, Upload
from django.views import generic
from django.views.generic.edit import CreateView
from django.urls import reverse_lazy
from django.contrib.auth.mixins import redirect_to_login, LoginRequiredMixin
from django.contrib.auth import login, authenticate
from django.contrib.auth.models import User
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.models import User
from .forms import SignUpForm
import openpyxl


# Create your views here.
def welcome(request):
    if Fish:
        num_fish = Fish.objects.all().count()
    if Upload:
        num_upload = Upload.objects.all().count()

    context = {
        'num_fish': num_fish,
        'num_upload': num_upload,
    }

    return render(request, 'welcome.html', context=context)


class PublicTable(LoginRequiredMixin, generic.ListView):
    model = Fish
    template_name = 'table/public_table.html'
    paginate_by = 50

    def get_queryset(self):
        new_context = fish_filter(self)
        return new_context


class Uploads(LoginRequiredMixin, generic.ListView):
    model = Upload
    template_name = 'table/uploads.html'
    paginate_by = 50

    def get_queryset(self):
        new_context = upload_filter(self)
        return new_context


class Account(LoginRequiredMixin, generic.ListView):
    model = Upload
    template_name = 'table/account.html'
    paginate_by = 50

    def get_queryset(self):
        new_context = upload_filter(self)
        return new_context


class Map(LoginRequiredMixin, generic.ListView):
    model = Fish
    template_name = 'table/map.html'


class Summary(LoginRequiredMixin, generic.ListView):
    model = Fish
    template_name = 'table/summary.html'

    def get_context_data(self, **kwargs):
        context = super(Summary, self).get_context_data(**kwargs)
        stats = summary_filter(self)
        context['stats'] = stats
        return context


class UserTable(LoginRequiredMixin, generic.ListView):
    model = Fish
    template_name = 'table/user_table.html'

    def get_queryset(self):
        new_context = fish_filter(self)
        return new_context


class FishCreate(LoginRequiredMixin, CreateView):
    model = Fish
    fields = ['species_name', 'fish_length', 'fish_weight',
              'date_caught', 'gps_coordinates', 'other_comments', 'upload']
    success_url = reverse_lazy('UserTable')

    def get_form(self, *args, **kwargs):
        form = super(FishCreate, self).get_form(*args, **kwargs)
        form.fields['upload'].queryset = Upload.objects.filter(uploader=self.request.user)
        return form


class UploadCreate(LoginRequiredMixin, CreateView):
    model = Upload
    fields = ['public', 'file_type', 'file']
    success_url = reverse_lazy('UserTable')

    # noinspection PyAttributeOutsideInit
    def form_valid(self, form):
        self.object = form.save()
        form.instance.uploader = self.request.user
        parse(self.request, self.object)
        return super(UploadCreate, self).form_valid(form)


class Stat:
    def __init__(self, fish_name, w, l, pk):
        self.name = fish_name
        self.count = 1
        self.total_w = w
        self.a_w = w
        self.total_l = l
        self.a_l = l
        self.pk_w = pk
        self.m_w = w
        self.pk_l = pk
        self.m_l = l


def summary_filter(self):
    order = self.request.GET.get('orderby', 'species_name')
    new_context = Fish.objects.order_by(order)
    stats = []
    si = 0
    prev_fish = "fake_fish"
    # if species name not in list, add to it, sorted by species name so know it's new fish when new one hit
    for i, f in enumerate(new_context):
        # if public
        if f.upload.public:
            # if on new fish
            cur_fish = f.species_name
            if i == 0 or cur_fish != prev_fish:
                stats.append(Stat(f.species_name, f.fish_weight, f.fish_length, f.pk))
                if i != 0:
                    si += 1
            # if still on same fish
            if cur_fish == prev_fish:
                stats[si].count += 1
                stats[si].total_w += f.fish_weight
                stats[si].a_w = stats[si].total_w / stats[si].count
                stats[si].total_l += f.fish_length
                stats[si].a_l = stats[si].total_l / stats[si].count
                if f.fish_weight > stats[si].m_w:
                    stats[si].pk_w = f.pk
                    stats[si].m_w = f.fish_weight
                if f.fish_length > stats[si].m_l:
                    stats[si].pk_l = f.pk
                    stats[si].m_l = f.fish_length
            prev_fish = f.species_name
    return stats


def upload_filter(self):
    filter_uploader = self.request.GET.get('filterUploader')
    filter_public = self.request.GET.get('filterPublic')
    order = self.request.GET.get('orderby', '-timestamp')
    new_context = Upload.objects.order_by(order)
    if filter_uploader:
        filter_id = User.objects.get(username=filter_uploader)
        new_context = new_context.filter(uploader=filter_id)
    if filter_public:
        new_context = new_context.filter(public=filter_public)
    return new_context


def fish_filter(self):
    filter_name = self.request.GET.get('filterName')
    filter_wmin = self.request.GET.get('filterWMin', default='0.0')
    filter_wmax = self.request.GET.get('filterWMax', default='100000')
    filter_lmin = self.request.GET.get('filterLMin', default='0.0')
    filter_lmax = self.request.GET.get('filterLMax', default='1000000')
    order = self.request.GET.get('orderby', '-date_caught')
    new_context = Fish.objects.order_by(order)
    if filter_name:
        new_context = new_context.filter(species_name=filter_name)
    if filter_wmin:
        new_context = new_context.filter(fish_weight__gte=filter_wmin)
    if filter_wmax:
        new_context = new_context.filter(fish_weight__lte=filter_wmax)
    if filter_lmin:
        new_context = new_context.filter(fish_length__gte=filter_lmin)
    if filter_lmax:
        new_context = new_context.filter(fish_length__lte=filter_lmax)
    return new_context


def parse(request, new_upload):
    if "GET" == request.method:
        return render(request, 'upload-create', {})
    else:
        excel_file = request.FILES['file']

        # you may put validations here to check extension or file size

        wb = openpyxl.load_workbook(excel_file)

        # select active sheet
        worksheet = wb["Sheet1"]
        # print(worksheet)

        # Insert each row as a Fish
        excel_data = list()
        for row in worksheet.iter_rows(min_row=2):
            new_fish = Fish()
            row_data = list()
            for cell in row:
                row_data.append(str(cell.value))
            new_fish.species_name = row_data[0]
            new_fish.fish_length = row_data[1]
            new_fish.fish_weight = row_data[2]
            new_fish.date_caught = row_data[3]
            new_fish.gps_coordinates = row_data[4]
            new_fish.other_comments = row_data[5]
            new_fish.upload = new_upload
            new_fish.save()
            excel_data.append(row_data)
        return


def signup_view(request):
    form = SignUpForm(request.POST)
    if form.is_valid():
        form.save()
        username = form.cleaned_data.get('username')
        password = form.cleaned_data.get('password1')
        user = authenticate(username=username, password=password)
        login(request, user)
        return redirect('welcome')
    else:
        form = SignUpForm()
    return render(request, 'signup.html', {'form': form})
